#include "DemoApp.h"
#include <fmod_errors.h>
#include <QtGlobal>

// for fmod error handling!
#define ERRCHECK(res)  if (res != FMOD_OK) { printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result)); exit(-1); }

DemoApp::DemoApp(int argc, char* argv[]) : QApplication(argc, argv) {
    qDebug()<<"DemoApp::DemoApp";

    QStringList cmdArgs;
    for (int i=0;i<argc;i++) cmdArgs.append(argv[i]);

    // create dialog window & show it
    mDemoDlg = new DemoDlgImp();
    mDemoDlg->show();

    // create set of records
    mRecordSet = new OOV::RecordSet(this);

    mRecordSet->setDataPath(":/Native-DrJordanCohenSRI");
    

    if (cmdArgs.count()>1) mRecordSet->setDataPath(cmdArgs[1]);
    else
    {
        // check available data directories
        QDir d;
        QStringList allDirs=d.entryList(QDir::AllDirs);
        // ignore . and .. and grab first real dir
        if (allDirs.count()==3)
        {
            mRecordSet->setDataPath(allDirs[2]);
        } else {
            // print modal msgbox to ask about which path to choose
            QString ds=QFileDialog::getOpenFileName(mDemoDlg,tr("SELECT DATA SET TO VISUALIZE"),QDir::currentPath(),tr("Data Set (rec.pau.mlf)"),NULL,QFileDialog::ShowDirsOnly|QFileDialog::ReadOnly|QFileDialog::DontUseNativeDialog);
            qDebug()<<ds.remove("/rec.pau.mlf");
            mRecordSet->setDataPath(ds);
        }
    }
    

    // connections
    connect(mDemoDlg->prevPB, SIGNAL(clicked()), this, SLOT(onPrevPressed()));
    connect(mDemoDlg->nextPB, SIGNAL(clicked()), this, SLOT(onNextPressed()));
    connect(mDemoDlg->recListWidget,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(recWordSelected(QListWidgetItem*)));
    connect(mDemoDlg->contextSlider,SIGNAL(valueChanged(int)),this,SLOT(onContextValueChanged(int)));

    connect(mDemoDlg->oovTB,SIGNAL(toggled(bool)),this,SLOT(updateClassification()));
    connect(mDemoDlg->ivincorrTB,SIGNAL(toggled(bool)),this,SLOT(updateClassification()));
    connect(mDemoDlg->misrecTB,SIGNAL(toggled(bool)),this,SLOT(updateClassification()));
    connect(mDemoDlg->ivcorrTB,SIGNAL(toggled(bool)),this,SLOT(updateClassification()));
    connect(mDemoDlg->silTB,SIGNAL(toggled(bool)),this,SLOT(updateClassification()));
    connect(mDemoDlg->corrTB,SIGNAL(toggled(bool)),this,SLOT(updateClassification()));

    connect(mDemoDlg->playWindowPB, SIGNAL(clicked()), this, SLOT(onPlayWinPressed()));
    connect(mDemoDlg->playUtterancePB, SIGNAL(clicked()), this, SLOT(onPlayUttPressed()));

    connect(mDemoDlg,SIGNAL(valueChanged()),this,SLOT(updateClassification()));
    connect(mDemoDlg,SIGNAL(taskChanged(OOV::Task)),this,SLOT(updateClassification()));

    init();

    // set active startup record!
    if (cmdArgs.count()>2) {
        int ar=QString(cmdArgs[2]).toInt();
        if (ar>0) mRecordSet->setActiveRecord(ar-1);
    }

    update();
}

void DemoApp::init() {
    //! init phnlist;

    QStringList phns;
    phns<<"aa"<<"ae"<<"ah"<<"ao"<<"aw"<<"ax"<<"axr"<<"ay"<<"b"<<"ch"<<"d"<<"dh"<<"eh"<<"el"<<"em"
        <<"en"<<"er"<<"ey"<<"f"<<"g"<<"hh"<<"ih"<<"iy"<<"jh"<<"k"<<"l"<<"m"<<"n"<<"ng"<<"ow"<<"oy"
        <<"p"<<"r"<<"s"<<"sh"<<"t"<<"th"<<"uh"<<"uw"<<"v"<<"w"<<"y"<<"z"<<"zh"<<"sil";

    for (int i=0;i<phns.count();i++){
        mPhones[phns[i]]=i;
    }

    // load rec wrd labels
    mRecordSet->loadMLF();

    // set default number of context frames to be displayed!
    setContextFrames();

    // try default name or '2k8.dct'
    mRecordSet->loadDictionary();

    QCompleter *wcompleter = new QCompleter(QStringList(mRecordSet->dictionary().keys()), this);
    wcompleter->setCaseSensitivity(Qt::CaseInsensitive);
    mDemoDlg->dctLineEdit->setCompleter(wcompleter);

    QCompleter *pcompleter = new QCompleter(QStringList(mRecordSet->dictionary().values()),this);
    pcompleter->setCaseSensitivity(Qt::CaseInsensitive);
    mDemoDlg->pronLineEdit->setCompleter(pcompleter);

    connect(mDemoDlg->dctLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(onDictionaryTextChanged(const QString&)));
    connect(mDemoDlg->pronLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(onPronTextChanged(const QString&)));


    //! load phn labels
    mRecordSet->loadMLF("rec.phn.mlf","recphn");
    mRecordSet->loadMLF("weak.phn.mlf","weakphn");

    //! load NN frame scores
    mRecordSet->loadFrameScores();

    //! load posterior images
    mRecordSet->loadFramePosteriors();

    //! FMOD init
    FMOD_RESULT result;
    mpFmodSound = NULL;

    result = FMOD_System_Create(&mpFmodSystem);
    ERRCHECK(result);

    result = FMOD_System_GetVersion(mpFmodSystem, &mFmodVersion);
    ERRCHECK(result);

    if (mFmodVersion < FMOD_VERSION)
    {
        qDebug() << tr("FMOD error! You are using an old version of FMOD ").append(mFmodVersion).append(". This program requires ").append(FMOD_VERSION);
        return;
    }

#ifndef WIN32
    result = FMOD_System_SetOutput(mpFmodSystem, FMOD_OUTPUTTYPE_ALSA);
    ERRCHECK(result);
#endif

    result = FMOD_System_Init(mpFmodSystem, 1, FMOD_INIT_NORMAL, 0);
    ERRCHECK(result);
}

void DemoApp::createLocalLabels() {
    //! CANT modify this in live mode!!!
    mAlignRecs=mRecordSet->getLabels();
}

bool DemoApp::skippableAlignWord(OOV::Label l) {
    QString w1=l.labelName.toUpper();
    return ((w1=="<S>")||(w1=="</S>")||(w1=="<PAU>"));
}

void DemoApp::displayRecord() {
    static int mLastActiveRecord=-1;
    const int ar=mRecordSet->activeRecord();

    // there is no active record!
    if (ar<0) return;

    // there is the same active record than in the last call!
    if (ar==mLastActiveRecord) return;

    mLastActiveRecord=ar;

    // wipe out old contents
    mDemoDlg->recListWidget->clear();

    // set new utterance name
    const QString* utt=mRecordSet->activeRecordName();

    if (utt)
        mDemoDlg->uttLabel->setText(QString("<b>%1</b>").arg(*utt));

    // pregenerate new local labels
    createLocalLabels();


    // fill the item list with words
    for (int i=0;i<mAlignRecs.size();i++) {
        QString recword=mAlignRecs[i].labelName;

        mDemoDlg->recListWidget->addItem(recword);

        // construct a tool tip with type/length of recognized word segment
        unsigned int start=mAlignRecs[i].start;
        unsigned int stop=mAlignRecs[i].stop;

        QString lenstr;
        for (unsigned int j=start;j<stop;j++) {
            lenstr.append("*");
        }

        while (lenstr.length()>35) {lenstr.replace(QRegExp("\\*\\*\\*\\*\\*\\.*\\*$"),"..*");};
        QString toolTip;

        if (mAlignRecs[i].duration()==0)
            toolTip.append(QString("<b>%1</b>").arg(0.01*start));
        else toolTip.append(QString("<b>%1%2%3</b>").arg(0.01*start).arg(lenstr).arg(0.01*stop));

        mDemoDlg->recListWidget->item(i)->setToolTip(toolTip);
    }
}

void DemoApp::displayRecognizedLabels() {
    const int w=screenWidth(0.65);
    const int h=screenHeight(0.12);
    QImage im(w,h,QImage::Format_ARGB32_Premultiplied);

    const int vMargin=0;
    const int hMargin=0;

    QRectF labelWin;
    labelWin.setX(hMargin);
    labelWin.setY(vMargin+mStrongPostWin.y());
    labelWin.setWidth(w-2*hMargin);
    labelWin.setHeight(h-2*vMargin);

    QRectF fromPostWin;
    fromPostWin.setWidth(contextFrames()*2);
    fromPostWin.setHeight(45);
    fromPostWin.moveCenter(QPoint(mRecordSet->activeFrame(),45*0.5));

    QPainter p;
    p.begin(&im);
    p.setRenderHint(QPainter::Antialiasing,true);
    p.fillRect(0,0,im.width(),im.height(),QBrush(QColor::fromRgbF(0,0,0,1)));

    QPen pen;
    pen.setJoinStyle(Qt::RoundJoin);
    //! draw scores
    QList <OOV::Score> sc=mRecordSet->getFrameScores();
    QPointF p0,p1;
    for (int i=1;i<sc.count();i++) {
        int offs=0;
        int hgt=36;
        //! draw sil prob
        if (mDemoDlg->silTB->isChecked()) {
            pen.setColor(Qt::blue);
            pen.setWidthF(2.5);
            p.setPen(pen);
            p0=winToWin(QPointF(i-1,offs+hgt*(1-sc[i-1].sil)),fromPostWin,labelWin);
            p1=winToWin(QPointF(i,offs+hgt*(1-sc[i].sil)),fromPostWin,labelWin);
            p.drawLine(QLineF(p0,p1));
        };

        //! draw ivcorr prob
        if (mDemoDlg->ivcorrTB->isChecked()) {
            pen.setColor(Qt::green);
            pen.setWidthF(2.5);
            p.setPen(pen);
            p0=winToWin(QPointF(i-1,offs+hgt*(1-sc[i-1].ivcorr)),fromPostWin,labelWin);
            p1=winToWin(QPointF(i,offs+hgt*(1-sc[i].ivcorr)),fromPostWin,labelWin);
            p.drawLine(QLineF(p0,p1));
        };

        //! draw ivincorr prob
        if (mDemoDlg->ivincorrTB->isChecked()) {
            pen.setColor(Qt::yellow);
            pen.setWidthF(2.5);
            p.setPen(pen);
            p0=winToWin(QPointF(i-1,offs+hgt*(1-sc[i-1].ivincorr)),fromPostWin,labelWin);
            p1=winToWin(QPointF(i,offs+hgt*(1-sc[i].ivincorr)),fromPostWin,labelWin);
            p.drawLine(QLineF(p0,p1));
        };

        //! draw misrec prob
        if (mDemoDlg->misrecTB->isChecked()) {
            pen.setColor(Qt::red);
            pen.setWidthF(2.5);
            p.setPen(pen);
            p0=winToWin(QPointF(i-1,offs+hgt*(1-sc[i-1].misrec())),fromPostWin,labelWin);
            p1=winToWin(QPointF(i,offs+hgt*(1-sc[i].misrec())),fromPostWin,labelWin);
            p.drawLine(QLineF(p0,p1));
            pen.setColor(Qt::yellow);
            pen.setWidth(1);
            pen.setStyle(Qt::DashLine);
            p.setPen(pen);
            p.drawLine(QLineF(p0,p1));
            pen.setWidthF(2.5);
            pen.setStyle(Qt::SolidLine);
        };

        //! draw corr prob
        if (mDemoDlg->corrTB->isChecked()) {
            pen.setColor(Qt::green);
            pen.setWidthF(2.5);
            p.setPen(pen);
            p0=winToWin(QPointF(i-1,offs+hgt*(1-sc[i-1].corr())),fromPostWin,labelWin);
            p1=winToWin(QPointF(i,offs+hgt*(1-sc[i].corr())),fromPostWin,labelWin);
            p.drawLine(QLineF(p0,p1));
            pen.setColor(Qt::blue);
            pen.setWidthF(1.5);
            pen.setStyle(Qt::DashLine);
            p.setPen(pen);
            p.drawLine(QLineF(p0,p1));
            pen.setStyle(Qt::SolidLine);
            pen.setWidthF(2.5);
        };

        //! draw oov prob
        if (mDemoDlg->oovTB->isChecked()) {
            pen.setColor(Qt::red);
            pen.setWidthF(2.5);
            p.setPen(pen);
            p0=winToWin(QPointF(i-1,offs+hgt*(1-sc[i-1].oov)),fromPostWin,labelWin);
            p1=winToWin(QPointF(i,offs+hgt*(1-sc[i].oov)),fromPostWin,labelWin);
            p.drawLine(QLineF(p0,p1));
        };
    }


    //! draw label names
    QList<OOV::Label> labs=mAlignRecs;

    for (int i=0;i<labs.count();i++) {
        double mid=labs[i].start+(labs[i].stop-labs[i].start)*0.5;

        p.save();
        QPointF p1=winToWin(QPointF(mid,0),fromPostWin,labelWin);
        QPointF topLeft=winToWin(QPointF(labs[i].start,0),fromPostWin,labelWin);

        p.setFont(QFont("Arial",12,QFont::Bold));

        // skip some of the trivial labels...
        if (skippableAlignWord(labs[i])) continue;

        QColor cl,tcl,cc;
        QPen classp;
        classp.setWidthF(2.5);

        //double corrScore=0;

        tcl=QColor::fromRgbF(1,1,1); // text color

        cc=classToColor(labs[i].rec);

        QRectF textRect(0,0,2*QRectF(topLeft,p1).width()-2,30);
        textRect.moveCenter(p1);

        cl.setAlphaF(0);

        p.setPen(Qt::transparent);
        p.setBrush(QBrush(cl,Qt::SolidPattern));
        p.drawRect(textRect);
        p.setBrush(cc);
        p.drawRect(QRectF(textRect.topLeft(),textRect.topLeft()+QPointF(textRect.width(),6)));
        p.setPen(tcl);
        double gap=qMax(0.0,30-textRect.width());
        textRect.setWidth(qMax(textRect.width(),60.0));
        textRect.moveCenter(p1-QPointF(gap,0));
        p.drawText(textRect,Qt::AlignCenter,labs[i].labelName.toUpper());
        p.restore();
    }

    p.end();
    mDemoDlg->recognizedLabel->setPixmap(QPixmap::fromImage(im));
}

QColor DemoApp::classToColor(OOV::Classification c) {
    switch(c) {
        case OOV::CLASS_Sil:
            return QColor::fromRgbF(0,0,1);
            break;
        case OOV::CLASS_IVCorr:
            return QColor::fromRgbF(0,0.7,0);
            break;
        case OOV::CLASS_IVIncorr:
            return QColor::fromRgbF(1,1,0);
            break;
        case OOV::CLASS_OOV:
            return QColor::fromRgbF(1,0,0);
            break;
        default:
        return QColor::fromRgbF(1,1,1);
        break;
    }
}

DemoApp::~DemoApp() {
    qDebug()<<"DemoApp::~DemoApp";
}


void DemoApp::onNextPressed() {
    qDebug()<<"next";
    mRecordSet->nextActiveRecord();
    update();
}

void DemoApp::onPrevPressed() {
    qDebug()<<"previous";
    mRecordSet->prevActiveRecord();
    update();
}

void DemoApp::recWordSelected(QListWidgetItem* it){
    unsigned int w=mDemoDlg->recListWidget->row(it);
    unsigned int start=mAlignRecs[w].start;
    unsigned int stop=mAlignRecs[w].stop;
    int middle=start+(stop-start)/2;

    qDebug()<<w<<":"<<start<<":"<<stop<<":"<<mAlignRecs[w].labelName;
    mRecordSet->setActiveFrame(middle);

    update();
}

QPointF DemoApp::winToWin(const QPointF& p, const QRectF& r1, const QRectF& r2) const {
    QPointF r;
    QPointF p1=r1.topLeft();
    QPointF p3=r2.topLeft();

    r.setX((p.x()-p1.x())*r2.width()/r1.width()+p3.x());
    r.setY((p.y()-p1.y())*r2.height()/r1.height()+p3.y());

    return r;
}

unsigned int DemoApp::screenWidth(double f) const {
    QDesktopWidget d;
    return qRound(f*d.availableGeometry().width());
}

unsigned int DemoApp::screenHeight(double f) const {
    QDesktopWidget d;
    return qRound(f*d.availableGeometry().height());
}

void DemoApp::displayPosteriors() {
    QTransform rot;
    rot.rotate(90);
    rot.scale(1,-1);

    QImage imWeak=mRecordSet->getWeakFramePosteriors()->transformed(rot);
    QImage imStrong=mRecordSet->getStrongFramePosteriors()->transformed(rot);
    imWeak.invertPixels();imStrong.invertPixels();

    QPixmap pmWeak=QPixmap::fromImage(imWeak);
    QPixmap pmStrong=QPixmap::fromImage(imStrong);

    const int w=screenWidth(0.65);
    const int h=screenHeight(0.40);
    QImage im(w,h,QImage::Format_ARGB32_Premultiplied);

    QPainter p;
    p.begin(&im);
    p.setRenderHint(QPainter::TextAntialiasing,true);
    p.fillRect(0,0,im.width(),im.height(),QBrush(QColor::fromRgbF(0.5,0.5,0.5,1)));

    const unsigned int hMargin=3;
    const unsigned int vMargin=13;

    mStrongPostWin.setX(hMargin);
    mStrongPostWin.setY(vMargin);
    mStrongPostWin.setWidth(w-2*hMargin);
    mStrongPostWin.setHeight(h/2-2*vMargin);

    mWeakPostWin.setX(hMargin);
    mWeakPostWin.setY(2*vMargin+mStrongPostWin.height()+mStrongPostWin.y());
    mWeakPostWin.setWidth(w-2*hMargin);
    mWeakPostWin.setHeight(h/2-2*vMargin);

    QRect fromPostWin;
    fromPostWin.setWidth(contextFrames()*2);
    fromPostWin.setHeight(pmStrong.height());
    fromPostWin.moveCenter(QPoint(mRecordSet->activeFrame()-1,pmStrong.height()*0.5));

    p.drawPixmap(mStrongPostWin,pmStrong,fromPostWin);
    p.drawPixmap(mWeakPostWin,pmWeak,fromPostWin);

    //! draw boundaries+words (rec)
    QColor col(255,0,0,80);
    p.setBrush(QBrush(col,Qt::NoBrush));
    int row=mDemoDlg->recListWidget->currentRow();
    QList<OOV::Label> labs=mAlignRecs;

    for (int i=0;i<labs.count();i++) {
        int pos=labs[i].start;

        QPointF p1=winToWin(QPointF(pos,0),fromPostWin,mStrongPostWin);
        QPointF p2=winToWin(QPointF(labs[i].stop,45),fromPostWin,mStrongPostWin);

        QPointF p3=winToWin(QPointF(pos,0),fromPostWin,mWeakPostWin);
        QPointF p4=winToWin(QPointF(labs[i].stop,45),fromPostWin,mWeakPostWin);
        if (row==i) {
            p.save();
            QPen pen;
            pen.setColor(Qt::red);
            pen.setWidth(4);
            p.setPen(pen);
            p.drawRect(QRectF(p1,p2));
            p.setPen(QPen(col,Qt::SolidLine));
            p.setFont(QFont("Arial",14));
            p.drawText(QRectF(p1,p2),Qt::AlignCenter,"SPECIFIC");
            p.drawText(QRectF(p3,p4),Qt::AlignCenter,"GENERAL");
            p.setPen(pen);
            p.drawRect(QRectF(p3,p4));
            p.restore();
        } else {
            p.setPen(Qt::gray);
            p.drawLine(p1.x(),p1.y(),p1.x(),p2.y());
            p.drawLine(p3.x(),p3.y(),p3.x(),p4.y());
            p.save();
            p.setPen(Qt::white);
            p.translate(p1);
            p.setFont(QFont("Arial",12,QFont::Bold));
            p.setBrush(QBrush(QColor::fromRgbF(0,1,0),Qt::SolidPattern));
            p.rotate(-90);
            QRectF r;
            //p.drawText(QPoint(-mStrongPostWin.height(),20),labs[i].labelName.toUpper());
            p.restore();
        }
    }

    //! draw recognized phone labels
    QList<OOV::Label> plabs=mRecordSet->getLabels("recphn");
    for (int i=0;i<plabs.count();i++) {
        int pos=plabs[i].start+(plabs[i].stop-plabs[i].start)/2;

        QString phn=plabs[i].labelName;
        if (mPhones.contains(phn)) {
            QPointF p0=winToWin(QPointF(pos,mPhones[phn]),fromPostWin,mStrongPostWin);
            p.save();
            p.translate(p0);
            p.setPen(Qt::blue);
            p.setFont(QFont("Arial",12,QFont::Bold));
            p.drawText(QPointF(0,-2),plabs[i].labelName);
            p.restore();
        }
    }

    //! draw weak phone labels
    plabs=mRecordSet->getLabels("weakphn");
    for (int i=0;i<plabs.count();i++) {
        int pos=plabs[i].start+(plabs[i].stop-plabs[i].start)/2;

        QPointF p0=winToWin(QPointF(pos,mPhones[plabs[i].labelName]),fromPostWin,mWeakPostWin);
        p.save();
        p.translate(p0);
        p.setPen(Qt::blue);
        p.setFont(QFont("Arial",12,QFont::Bold));
        p.drawText(QPointF(0,-2),plabs[i].labelName);
        p.restore();
    }

    p.end();

    mDemoDlg->mainPostLabel->setPixmap(QPixmap::fromImage(im));
}

void DemoApp::setContextFrames(unsigned int numFrames) {
    mContextFrames=numFrames;
    mContextTimer.setInterval((numFrames*2+1)*10);
}

unsigned int DemoApp::contextFrames() const {
    return mContextFrames;
}

void DemoApp::updateClassification() {
    classify();
    displayRecognizedLabels();
}

void DemoApp::update() {
    displayRecord();
    classify();

    displayPosteriors();
    displayRecognizedLabels();

    qDebug() << "update FileNrLabel";
    int numFiles=mRecordSet->recordCount();
    if (!numFiles) return;
    int currentFile=mRecordSet->activeRecord()+1;
    mDemoDlg->fileNrGroupBox->setTitle(QString("%1 of %2").arg(currentFile).arg(numFiles));

    loadSound();
}

void DemoApp::onContextValueChanged(int v) {
    qDebug() << "zoom in/out";
    setContextFrames(v);
    update();
}

void DemoApp::loadSound() {
    FMOD_RESULT result;
    FMOD_CREATESOUNDEXINFO exinfo;

    QByteArray data;

    QFile f(mRecordSet->activeSound());
    f.open(QIODevice::ReadOnly);
    data=f.readAll();
    f.close();

    memset(&exinfo, 0, sizeof(FMOD_CREATESOUNDEXINFO));
    exinfo.cbsize = sizeof(FMOD_CREATESOUNDEXINFO);
    exinfo.length = data.size();

    if (mpFmodSound) {
        result = FMOD_Sound_Release(mpFmodSound);
        ERRCHECK(result);
    }

    //qDebug()<<data.size();
    //qDebug()<<mRecordSet->activeSound();

    result=FMOD_System_CreateSound(mpFmodSystem,data.data(),FMOD_HARDWARE|FMOD_OPENMEMORY,&exinfo,&mpFmodSound);
    ERRCHECK(result);

    result=FMOD_Sound_SetMode(mpFmodSound,FMOD_LOOP_OFF);
    ERRCHECK(result);
}

void DemoApp::onPlayUttPressed() {
    qDebug() << "audioFile";
    FMOD_RESULT result;

    if (mpFmodSound) {
        result = FMOD_System_PlaySound(mpFmodSystem, FMOD_CHANNEL_FREE, mpFmodSound, 0, &mpFmodChannel);
        ERRCHECK(result);
    }
}

void DemoApp::onPlayWinPressed() {
    qDebug() << "play win only";
    FMOD_RESULT result;

    if (mpFmodSound) {
        result = FMOD_System_PlaySound(mpFmodSystem, FMOD_CHANNEL_FREE, mpFmodSound, 0, &mpFmodChannel);
        ERRCHECK(result);
    }

    int start=(mRecordSet->activeFrame()-contextFrames())*10;
    unsigned int trunkedStart=qMax(0,start);
    unsigned int duration=(contextFrames()*2+1)*10+qMin(start,0);
    qDebug() << start;

    FMOD_Channel_SetPosition(mpFmodChannel, trunkedStart, FMOD_TIMEUNIT_MS);
    QTimer::singleShot(duration,this,SLOT(stopPlayback()));
}


void DemoApp::stopPlayback() {
    FMOD_Channel_Stop(mpFmodChannel);
}

void DemoApp::classify() {
    //qDebug()<<"creating reference classification";
    //int j=0;

    // average NN scores over words...
    QList<OOV::Score> sc=mRecordSet->getFrameScores();

    for (int i=0;i<mAlignRecs.count();i++) {
        double avgSil,avgIVCorr,avgIVIncorr,avgOOV;

        avgSil=avgIVCorr=avgIVIncorr=avgOOV=0;

        // skip empty labels with 0 duration
        if (mAlignRecs[i].duration()==0) continue;

        for (unsigned int j=mAlignRecs[i].start;j<mAlignRecs[i].stop;j++) {
            avgSil+=sc[j].sil;
            avgIVCorr+=sc[j].ivcorr;
            avgIVIncorr+=sc[j].ivincorr;
            avgOOV+=sc[j].oov;
        }
        avgSil/=mAlignRecs[i].duration();
        avgIVCorr/=mAlignRecs[i].duration();
        avgIVIncorr/=mAlignRecs[i].duration();
        avgOOV/=mAlignRecs[i].duration();

        mAlignRecs[i].avgScore.sil=avgSil;
        mAlignRecs[i].avgScore.ivcorr=avgIVCorr;
        mAlignRecs[i].avgScore.ivincorr=avgIVIncorr;
        mAlignRecs[i].avgScore.oov=avgOOV;
    }

    // classifying with the selected NN task
    for (int i=0;i<mAlignRecs.count();i++) {
        mAlignRecs[i].rec=classifyLabel(mAlignRecs[i]);
    }

    // setting recognition icons
    for (int i=0;i<mAlignRecs.count();i++) {
        mDemoDlg->recListWidget->item(i)->setIcon(getClassIcon(mAlignRecs[i].rec));
    }
}

OOV::Classification DemoApp::classifyLabel(const OOV::Label& l) {
    OOV::Classification c;

    if (l.duration()==0) return OOV::CLASS_None;

    switch (mDemoDlg->activeTask()) {
        case OOV::TASK_FullClassification: {
            double s[]={l.avgScore.sil,l.avgScore.ivcorr,l.avgScore.ivincorr,l.avgScore.oov};
            if (s[0]>=s[1]&&s[0]>=s[2]&&s[0]>=s[3]) c=OOV::CLASS_Sil;
            if (s[1]>=s[0]&&s[1]>=s[2]&&s[1]>=s[3]) c=OOV::CLASS_IVCorr;
            if (s[2]>=s[0]&&s[2]>=s[1]&&s[2]>=s[3]) c=OOV::CLASS_IVIncorr;
            if (s[3]>=s[0]&&s[3]>=s[1]&&s[3]>=s[2]) c=OOV::CLASS_OOV;
            break;
        };
        case OOV::TASK_TriClassification: {
            double p[3];
            mDemoDlg->triangleInput->getPriors(p[1],p[2],p[0]);
            double s[]={p[0]*(l.avgScore.sil+l.avgScore.ivcorr),p[1]*l.avgScore.ivincorr,p[2]*l.avgScore.oov};
            if (s[0]>=s[1]&&s[0]>=s[2]) c=OOV::CLASS_Corr;
            if (s[1]>=s[0]&&s[1]>=s[2]) c=OOV::CLASS_IVIncorr;
            if (s[2]>=s[0]&&s[2]>=s[1]) c=OOV::CLASS_OOV;
            break;
        };
        case OOV::TASK_OOV_NoOOV: {
            if (l.avgScore.oov>=mDemoDlg->SB_2->value()) {
                c=OOV::CLASS_OOV;
            } else {
                c=OOV::CLASS_NoOOV;
            }
            break;
        };
        case OOV::TASK_MisrecCorrect: {
            if (l.avgScore.misrec()>=mDemoDlg->SB_3->value()) {
                c=OOV::CLASS_Misrec;
            } else {
                c=OOV::CLASS_Corr;
            }
            break;
        };
        case OOV::TASK_OOV_IV_given_Misrec: {
            double t=mDemoDlg->SB_4->value();
            //if (l.avgScore.oov*(1-t)>l.avgScore.ivincorr*t) {
            if (l.avgScore.oov/l.avgScore.misrec()>=t) {
                c=OOV::CLASS_OOV;
            } else {
                c=OOV::CLASS_IVIncorr;
            }
            break;
        };
        default:
            c=OOV::CLASS_None;
            break;
    };

    return c;
}

const QIcon& DemoApp::getClassIcon(OOV::Classification c){
    static bool mIconsLoaded = false;
    static QIcon mClassIcons[OOV::CLASS_MAX];

    if (!mIconsLoaded) {
        QSize s(30,30);

        mClassIcons[0].addFile(":/class/unk.png",s,QIcon::Normal);
        mClassIcons[0].addFile(":/class/unk.png",s,QIcon::Selected);
        mClassIcons[1].addFile(":/class/sil.png",s,QIcon::Selected);
        mClassIcons[1].addFile(":/class/sil.png",s,QIcon::Normal);
        mClassIcons[2].addFile(":/class/ivcorr.png",s,QIcon::Selected);
        mClassIcons[2].addFile(":/class/ivcorr.png",s,QIcon::Normal);
        mClassIcons[3].addFile(":/class/ivincorr.png",s,QIcon::Selected);
        mClassIcons[3].addFile(":/class/ivincorr.png",s,QIcon::Normal);
        mClassIcons[4].addFile(":/class/oov.png",s,QIcon::Selected);
        mClassIcons[4].addFile(":/class/oov.png",s,QIcon::Normal);
        mClassIcons[5].addFile(":/class/notoov.png",s,QIcon::Selected);
        mClassIcons[5].addFile(":/class/notoov.png",s,QIcon::Normal);
        mClassIcons[6].addFile(":/class/corr.png",s,QIcon::Selected);
        mClassIcons[6].addFile(":/class/corr.png",s,QIcon::Normal);
        mClassIcons[7].addFile(":/class/misrec.png",s,QIcon::Selected);
        mClassIcons[7].addFile(":/class/misrec.png",s,QIcon::Normal);

        mIconsLoaded=true;
    }
    return mClassIcons[c];
}

void DemoApp::onDictionaryTextChanged(const QString& s) {
    mDemoDlg->pronLineEdit->setText(mRecordSet->dictionary()[s]);
}

void DemoApp::onPronTextChanged(const QString&) {
//    mDemoDlg->dctLineEdit->setText("vfdv");
}

