#ifndef __OOV_H__
#define __OOV_H__

#include <QtCore>
#include <QtGui>

namespace OOV {

    //! any possible classification
    typedef enum {
        CLASS_None=0,
        CLASS_Sil,
        CLASS_IVCorr,
        CLASS_IVIncorr,
        CLASS_OOV,
        CLASS_NoOOV,
        CLASS_Corr,
        CLASS_Misrec,
        CLASS_MAX
    } Classification;

    //! any possible tasks
    typedef enum {
        TASK_FullClassification=0,
        TASK_TriClassification,
        TASK_OOV_NoOOV,
        TASK_MisrecCorrect,
        TASK_OOV_IV_given_Misrec,
        TASK_MAX
    } Task;

    //! stores a score for a unit
    struct Score {
        Score() {sil=ivcorr=ivincorr=oov=0;};
        double sil;
        double ivcorr;
        double ivincorr;
        double oov;
        double corr() const { return ivcorr+sil; };
        double misrec() const { return ivincorr+oov; };
        double nooov() const { return 1.0-oov; };
        double fromClass(OOV::Classification c) {
            switch (c) {
                case CLASS_Sil: return sil;
                case CLASS_IVCorr: return ivcorr;
                case CLASS_IVIncorr: return ivincorr;
                case CLASS_OOV: return oov;
                case CLASS_NoOOV: return nooov();
                case CLASS_Corr: return corr();
                case CLASS_Misrec: return misrec();
                default:
                return 0;
            }
        }
    };
    
    //! represents a label with corresponding data
    struct Label {
        Label() {labelName="none";start=0;stop=0;ref=rec=CLASS_None;}
        QString labelName;

        unsigned int start;
        unsigned int stop;

        unsigned int duration() const {return stop-start; };
        unsigned int mid() const {return start+duration()/2;};
        bool contains(unsigned int pos) const { return ((start<=pos)&&(stop>pos)); }
        bool isEmpty() const { return (start==stop); };

        bool correctlyClassified() {return (ref==rec);};

        Classification ref, rec;

        Score avgScore;
    };

    //! represents a record with corresponding data
    struct Record {
        Record() {clear();};
        QString recordName;

        QMap < QString, QList <Label> > labels;

        //! call this before re-using a record!!!
        void clear() { 
            activeFrame=-1;
            audioFile="";
            recordName="none";
            scores.clear();
            strongposts=weakposts=QImage();
        };

        QList<Score> scores;
        QImage strongposts;
        QImage weakposts;
        QString audioFile;

        int activeFrame;

        unsigned int length() const {
            return scores.count(); 
        };

        int labelIndex(const QString& hash, int frmpos) const {
            if (frmpos<0) return frmpos;
            int i;
            for (i=0;i<labels[hash].count();i++) {
                if (labels[hash][i].contains(frmpos)) return i;
            }
            return -1;
        };

        int activeLabelIndex(const QString& hash) const { 
            return labelIndex(hash,activeFrame);
        };
    };


    class RecordSet : public QObject {
      Q_OBJECT
    public:
        //! constructor
        RecordSet( QObject* parent = 0);

        //! destructor
        virtual ~RecordSet();

        //! set data path
        void setDataPath(const QString& path="data");

        //! get data path
        const QString& dataPath() const;

        //! load OOV dictionary
        bool loadDictionary( const QString& dctFN = "dict.dct" );

        //! set currently active record
        //! below zero means none is active
        void setActiveRecord(int rec);

        void nextActiveRecord();
        void prevActiveRecord();

        //! returns currently active record
        //! below zero means none is active
        int activeRecord() const;


        //! returns currently active record name
        //! null ptr if none is active
        const QString* activeRecordName() const;



        //! set currently active frame of active record
        //! below zero means none is active
        void setActiveFrame(int fr);

        //! returns currently active frame of active record
        //! below zero means none is active
        int activeFrame() const;



        //! returns reference to the labels of the active record stored under the given hash (default "wordrec")
        //! and in case no such hash is stored an empty list
        QList <Label> getLabels( const QString& hash = "wordrec") const;


        //! loads an with SRover accordingly timed and aligned MLF
        //! using given hashes (default "wordref" and "wordrec")
        bool loadAlignedMLF(const QString& fileName = "aligned.mlf", const QString& h1 = "wordrec", const QString& h2="wordref");


        //! loads an MLF using given hash (default "wordrec")
        bool loadMLF(const QString& fileName = "rec.pau.mlf", const QString& hash = "wordrec");

        //! loads frame-level score files for the record set with given path/ext
        //! %1 is a placeholder for the record name
        bool loadFrameScores(const QString& wildcard="class/%1.asci");

        //! load frame-level posterior files for the record set with given path/ext
        //! %1 is a placeholder for the record name
        bool loadFramePosteriors(const QString& wildcard="joinpost/%1.%2.pgm");
        
        //! return frame scores of the active record
        QList<Score> getFrameScores();

        //! return reference of posteriors image of the active record
        const QImage* getStrongFramePosteriors();
        const QImage* getWeakFramePosteriors();

        //! returns active label (given active record/frame position/label hash)
        //! null ptr if no label is active
        const Label* activeLabel( const QString& hash = "wordrec" ) const;

        //! returns number of records in the set
        unsigned int recordCount() const;

        //! return number of frames in the active record
        unsigned int frameCount() const;
        
        //! returns sound file for active record
        QString activeSound() const;

        //! classifies based on the scores the currently active record
        //! given the three thresholds for sil/misrec/oov (all within 0...1)
        void classify(double sil,double misrec,double oov,const QString& hash="wordrec");

        //! returns dictionary
        QMap <QString,QString> dictionary() const;


      private:
        QVector<unsigned int> mOOVEval,mMisrecEval,mSilEval;

        QString mDataPath;

        QMap <QString,Record> mRecordSet;
        QStringList mRecordName;
        int mActiveRecord;

        QMap <QString,QString> mDictionary;
    };
    
}

#endif // __OOV_H__
